@@ -61,6 +61,10 @@ struct MathButtonsPrivate
     GtkWidget *bit_panel;
     GtkWidget *bit_labels[MAXBITS];
 
+    GtkWidget *source_currency_combo;
+    GtkWidget *target_currency_combo;
+    GtkWidget *currency_label;
+
     GtkWidget *character_code_dialog;
     GtkWidget *character_code_entry;
 };
@@ -102,7 +106,7 @@ static ButtonData button_data[] = {
       N_("Pi [Ctrl+P]")},
     {"eulers_number",      "e", NUMBER,
       /* Tooltip for the Euler's Number button */
-      N_("Eulers Number")},
+      N_("Euler’s Number")},
     {"imaginary",          "i", NUMBER, NULL},
     {"numeric_point", NULL, NUMBER, NULL},
     {"subscript", NULL, NUMBER_BOLD,
@@ -110,7 +114,7 @@ static ButtonData button_data[] = {
       N_("Subscript mode [Alt]")},
     {"superscript", NULL, NUMBER_BOLD,
       /* Tooltip for the superscript button */
-      N_("Supercript mode [Ctrl]")},
+      N_("Superscript mode [Ctrl]")},
     {"exponential", NULL, NUMBER_BOLD,
       /* Tooltip for the scientific exponent button */
       N_("Scientific exponent [Ctrl+E]")},
@@ -282,9 +286,6 @@ static ButtonData button_data[] = {
     {"finc_gross_profit_margin", NULL, FUNCTION,
       /* Tooltip for the gross profit margin button */
       N_("Gross Profit Margin")},
-    {"currency", NULL, FUNCTION,
-      /* Tooltip for the currency button */
-      N_("Currency Converter")},
     {NULL, NULL, 0, NULL}
 };
 
@@ -364,12 +365,6 @@ static void
 load_finc_dialogs(MathButtons *buttons)
 {
     int i, j;
-    GtkListStore *currency_store;
-    GtkCellRenderer *render;
-    GtkSpinButton *currency_amount_upper;
-    GtkSpinButton *currency_amount_lower;
-    GtkComboBox   *currency_type_upper;
-    GtkComboBox   *currency_type_lower;
 
     set_int_data(buttons->priv->financial_ui, "ctrm_dialog", "finc_dialog", FINC_CTRM_DIALOG);
     set_int_data(buttons->priv->financial_ui, "ddb_dialog", "finc_dialog", FINC_DDB_DIALOG);
@@ -392,167 +387,189 @@ load_finc_dialogs(MathButtons *buttons)
             g_object_set_data(o, "finc_dialog", GINT_TO_POINTER(i));
         }
     }
+}
 
-    currency_amount_upper = GTK_SPIN_BUTTON(gtk_builder_get_object(buttons->priv->financial_ui, "currency_amount_upper"));
-    currency_amount_lower = GTK_SPIN_BUTTON(gtk_builder_get_object(buttons->priv->financial_ui, "currency_amount_lower"));
-    currency_type_upper = GTK_COMBO_BOX(gtk_builder_get_object(buttons->priv->financial_ui, "currency_type_upper"));
-    currency_type_lower = GTK_COMBO_BOX(gtk_builder_get_object(buttons->priv->financial_ui, "currency_type_lower"));
 
-    currency_store = gtk_list_store_new(2,
-                                        G_TYPE_INT,
-                                        G_TYPE_STRING);
+static void
+update_angle_label (MathButtons *buttons)
+{
+    MPNumber x;
+    MPNumber pi, max_value, min_value, fraction, input, output;
+    char *label, input_text[1024], output_text[1024];
 
-    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(currency_store),
-                                         1,
-                                         GTK_SORT_ASCENDING);
+    if (!buttons->priv->angle_label)
+        return;
 
-    gtk_combo_box_set_model(currency_type_upper, GTK_TREE_MODEL(currency_store));
-    gtk_combo_box_set_model(currency_type_lower, GTK_TREE_MODEL(currency_store));
+    if (!math_equation_get_number(buttons->priv->equation, &x))
+        return;
 
-    render = gtk_cell_renderer_text_new();
+    mp_get_pi(&pi);
+    switch (math_equation_get_angle_units(buttons->priv->equation)) {
+    default:
+    case MP_DEGREES:
+        label = g_strdup("");
+        break;
+    case MP_RADIANS:
+        /* Clip to the range ±2π */
+        mp_multiply_integer(&pi, 2, &max_value);
+        mp_invert_sign(&max_value, &min_value);
+        if (!mp_is_equal(&x, &max_value) && !mp_is_equal(&x, &min_value)) {
+            mp_divide(&x, &max_value, &fraction);
+            mp_fractional_component(&fraction, &fraction);
+            mp_multiply(&fraction, &max_value, &input);
+        }
+        else {
+            mp_set_from_mp(&x, &input);
+            mp_set_from_integer(mp_is_negative(&input) ? -1 : 1, &fraction);
+        }
+        mp_cast_to_string(&input, 10, 10, 2, false, '.', input_text, 1024);
 
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(currency_type_upper),
-                               render,
-                               TRUE);
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(currency_type_lower),
-                               render,
-                               TRUE);
+        mp_multiply_integer(&fraction, 360, &output);
+        mp_cast_to_string(&output, 10, 10, 2, false, '.', output_text, 1024);
+        label = g_strdup_printf(_("%s radians = %s degrees"), input_text, output_text);
+        break;
+    case MP_GRADIANS:
+        /* Clip to the range ±400 */
+        mp_set_from_integer(400, &max_value);
+        mp_invert_sign(&max_value, &min_value);
+        if (!mp_is_equal(&x, &max_value) && !mp_is_equal(&x, &min_value)) {
+            mp_divide(&x, &max_value, &fraction);
+            mp_fractional_component(&fraction, &fraction);
+            mp_multiply(&fraction, &max_value, &input);
+        }
+        else {
+            mp_set_from_mp(&x, &input);
+            mp_set_from_integer(mp_is_negative(&input) ? -1 : 1, &fraction);
+        }
 
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(currency_type_upper),
-                                  render,
-                                  "text",
-                                  1);
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(currency_type_lower),
-                                  render,
-                                  "text",
-                                  1);
+        mp_cast_to_string(&input, 10, 10, 2, false, '.', input_text, 1024);
 
-    set_int_data(buttons->priv->financial_ui, "currency_amount_upper", "target", CURRENCY_TARGET_LOWER);
-    set_int_data(buttons->priv->financial_ui, "currency_amount_lower", "target", CURRENCY_TARGET_UPPER);
+        mp_multiply_integer(&fraction, 360, &output);
+        mp_cast_to_string(&output, 10, 10, 2, false, '.', output_text, 1024);
+        label = g_strdup_printf(_("%s gradians = %s degrees"), input_text, output_text);
+        break;
+    }
+
+    gtk_label_set_text(GTK_LABEL(buttons->priv->angle_label), label);
+    g_free(label);
+}
+
+
+static void
+update_bit_panel(MathButtons *buttons)
+{
+    MPNumber x;
+    gboolean enabled;
+    guint64 bits;
+    int i;
+    GString *label;
+    gint base;
+
+    if (!buttons->priv->bit_panel)
+        return;
+  
+    enabled = math_equation_get_number(buttons->priv->equation, &x);
+
+    if (enabled) {
+        MPNumber max, fraction;
+
+        mp_set_from_unsigned_integer(G_MAXUINT64, &max);
+        mp_fractional_part(&x, &fraction);
+        if (mp_is_negative(&x) || mp_is_greater_than(&x, &max) || !mp_is_zero(&fraction))
+            enabled = FALSE;
+        else
+            bits = mp_cast_to_unsigned_int(&x);
+    }
+
+    gtk_widget_set_sensitive(buttons->priv->bit_panel, enabled);
+    gtk_widget_set_sensitive(buttons->priv->base_label, enabled);
+      
+    if (!enabled)
+        return;
+
+    for (i = 0; i < MAXBITS; i++) {
+        const gchar *label;
+
+        if (bits & (1LL << (MAXBITS-i-1)))
+            label = " 1";
+        else
+            label = " 0";
+        gtk_label_set_text(GTK_LABEL(buttons->priv->bit_labels[i]), label);
+    }
+
+    base = math_equation_get_base(buttons->priv->equation);      
+    label = g_string_new("");
+    if (base != 8) {
+        if (label->len != 0)
+            g_string_append(label, " = ");
+        g_string_append_printf(label, "%lo", bits);
+        g_string_append(label, "₈");
+    }
+    if (base != 10) {
+        if (label->len != 0)
+            g_string_append(label, " = ");
+        g_string_append_printf(label, "%lu", bits);
+        g_string_append(label, "₁₀");
+    }
+    if (base != 16) {
+        if (label->len != 0)
+            g_string_append(label, " = ");
+        g_string_append_printf(label, "%lX", bits);
+        g_string_append(label, "₁₆");
+    }
+
+    gtk_label_set_text(GTK_LABEL(buttons->priv->base_label), label->str);
+    g_string_free(label, TRUE);
+}
+
+
+static void
+update_currency_label(MathButtons *buttons)
+{
+    MPNumber x, value;
+    char *label;
+
+    if (!buttons->priv->currency_label)
+        return;
+
+    if (!math_equation_get_number(buttons->priv->equation, &x))
+        return;
+  
+    if (currency_convert(&x,
+                         math_equation_get_source_currency(buttons->priv->equation),
+                         math_equation_get_target_currency(buttons->priv->equation),
+                         &value)) {
+        char input_text[1024], output_text[1024];
+        const char *source_symbol, *target_symbol;
+        int i;
+
+        mp_cast_to_string(&x, 10, 10, 2, false, '.', input_text, 1024);
+        mp_cast_to_string(&value, 10, 10, 2, false, '.', output_text, 1024);
+
+        for (i = 0; strcmp(math_equation_get_source_currency(buttons->priv->equation), currency_names[i].short_name) != 0; i++);
+        source_symbol = currency_names[i].symbol;
+        for (i = 0; strcmp(math_equation_get_target_currency(buttons->priv->equation), currency_names[i].short_name) != 0; i++);
+        target_symbol = currency_names[i].symbol;
+
+        /* Translators: first and third %s are currency symbols, second and fourth are amounts in these currencies, you may want to change the order of these, example: $100 = €100 */
+        label = g_strdup_printf(_("%s%s = %s%s"),
+                                source_symbol, input_text,
+                                target_symbol, output_text);
+    }
+    else
+        label = g_strdup("");
+
+    gtk_label_set_text(GTK_LABEL(buttons->priv->currency_label), label);
+    g_free(label);
 }
 
 
 static void
 display_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
 {
-    MPNumber x;
-    gboolean is_number;
-
-    is_number = math_equation_get_number(equation, &x);
-
-    if (buttons->priv->angle_label && is_number) {
-        MPNumber pi, max_value, min_value, fraction, input, output;
-        char *label, input_text[1024], output_text[1024];
-
-        mp_get_pi(&pi);
-        switch (math_equation_get_angle_units(equation)) {
-        default:
-        case MP_DEGREES:
-            label = g_strdup("");
-            break;
-        case MP_RADIANS:
-            /* Clip to the range ±2π */
-            mp_multiply_integer(&pi, 2, &max_value);
-            mp_invert_sign(&max_value, &min_value);
-            if (!mp_is_equal(&x, &max_value) && !mp_is_equal(&x, &min_value)) {
-                mp_divide(&x, &max_value, &fraction);
-                mp_fractional_component(&fraction, &fraction);
-                mp_multiply(&fraction, &max_value, &input);
-            }
-            else {
-                mp_set_from_mp(&x, &input);
-                mp_set_from_integer(mp_is_negative(&input) ? -1 : 1, &fraction);
-            }
-            mp_cast_to_string(&input, 10, 10, 2, false, input_text, 1024);
-
-            mp_multiply_integer(&fraction, 360, &output);
-            mp_cast_to_string(&output, 10, 10, 2, false, output_text, 1024);
-            label = g_strdup_printf("%s radians = %s degrees", input_text, output_text);
-            break;
-        case MP_GRADIANS:
-            /* Clip to the range ±400 */
-            mp_set_from_integer(400, &max_value);
-            mp_invert_sign(&max_value, &min_value);
-            if (!mp_is_equal(&x, &max_value) && !mp_is_equal(&x, &min_value)) {
-                mp_divide(&x, &max_value, &fraction);
-                mp_fractional_component(&fraction, &fraction);
-                mp_multiply(&fraction, &max_value, &input);
-            }
-            else {
-                mp_set_from_mp(&x, &input);
-                mp_set_from_integer(mp_is_negative(&input) ? -1 : 1, &fraction);
-            }
-
-            mp_cast_to_string(&input, 10, 10, 2, false, input_text, 1024);
-
-            mp_multiply_integer(&fraction, 360, &output);
-            mp_cast_to_string(&output, 10, 10, 2, false, output_text, 1024);
-            label = g_strdup_printf("%s gradians = %s degrees", input_text, output_text);
-            break;
-        }
-
-        gtk_label_set_text(GTK_LABEL(buttons->priv->angle_label), label);
-        g_free(label);
-    }
-  
-    if (buttons->priv->bit_panel) {
-        gboolean enabled = is_number;
-        guint64 bits;
-
-        if (enabled) {
-            MPNumber max, fraction;
-
-            mp_set_from_unsigned_integer(G_MAXUINT64, &max);
-            mp_fractional_part(&x, &fraction);
-            if (mp_is_negative(&x) || mp_is_greater_than(&x, &max) || !mp_is_zero(&fraction))
-                enabled = FALSE;
-            else
-                bits = mp_cast_to_unsigned_int(&x);
-        }
-
-        gtk_widget_set_sensitive(buttons->priv->bit_panel, enabled);
-        gtk_widget_set_sensitive(buttons->priv->base_label, enabled);
-      
-        if (enabled) {
-            int i;
-            GString *label;
-            gint base;
-
-            for (i = 0; i < MAXBITS; i++) {
-                const gchar *label;
-
-                if (bits & (1LL << (MAXBITS-i-1)))
-                    label = " 1";
-                else
-                    label = " 0";
-                gtk_label_set_text(GTK_LABEL(buttons->priv->bit_labels[i]), label);
-            }
-
-            base = math_equation_get_base(equation);      
-            label = g_string_new("");
-            if (base != 8) {
-                if (label->len != 0)
-                    g_string_append(label, " = ");
-                g_string_append_printf(label, "%lo", bits);
-                g_string_append(label, "₈");
-            }
-            if (base != 10) {
-                if (label->len != 0)
-                    g_string_append(label, " = ");
-                g_string_append_printf(label, "%lu", bits);
-                g_string_append(label, "₁₀");
-            }
-            if (base != 16) {
-                if (label->len != 0)
-                    g_string_append(label, " = ");
-                g_string_append_printf(label, "%lX", bits);
-                g_string_append(label, "₁₆");
-            }
-
-            gtk_label_set_text(GTK_LABEL(buttons->priv->base_label), label->str);
-            g_string_free(label, TRUE);
-        }
-    }
+    update_angle_label(buttons);
+    update_currency_label(buttons);
+    update_bit_panel(buttons);
 }
 
 
@@ -639,6 +656,102 @@ base_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
 }
 
 
+static void
+source_currency_combo_changed_cb(GtkWidget *combo, MathButtons *buttons)
+{
+    gchar *value;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
+    gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter);
+    gtk_tree_model_get(model, &iter, 0, &value, -1);
+
+    math_equation_set_source_currency(buttons->priv->equation, value);
+    g_free (value);
+}
+
+
+static void
+source_currency_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gboolean valid;
+  
+    if (buttons->priv->mode != FINANCIAL)
+        return;
+
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->source_currency_combo));
+    valid = gtk_tree_model_get_iter_first(model, &iter);
+
+    while (valid) {
+        gchar *v;
+        gboolean matched;
+
+        gtk_tree_model_get(model, &iter, 0, &v, -1);
+        matched = strcmp (math_equation_get_source_currency(buttons->priv->equation), v) == 0;
+        g_free (v);
+        if (matched)
+            break;
+        valid = gtk_tree_model_iter_next(model, &iter);
+    }
+    if (!valid)
+        valid = gtk_tree_model_get_iter_first(model, &iter);
+
+    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->source_currency_combo), &iter);
+    update_currency_label(buttons);
+}
+
+
+static void
+target_currency_combo_changed_cb(GtkWidget *combo, MathButtons *buttons)
+{
+    gchar *value;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
+    gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter);
+    gtk_tree_model_get(model, &iter, 0, &value, -1);
+
+    math_equation_set_target_currency(buttons->priv->equation, value);
+    g_free (value);
+}
+
+
+static void
+target_currency_changed_cb(MathEquation *equation, GParamSpec *spec, MathButtons *buttons)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gboolean valid;
+  
+    if (buttons->priv->mode != FINANCIAL)
+        return;
+
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(buttons->priv->target_currency_combo));
+    valid = gtk_tree_model_get_iter_first(model, &iter);
+
+    while (valid) {
+        gchar *v;
+        gboolean matched;
+
+        gtk_tree_model_get(model, &iter, 0, &v, -1);
+        matched = strcmp (math_equation_get_target_currency(buttons->priv->equation), v) == 0;
+        g_free (v);
+        if (matched)
+            break;
+        valid = gtk_tree_model_iter_next(model, &iter);
+    }
+    if (!valid)
+        valid = gtk_tree_model_get_iter_first(model, &iter);
+
+    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(buttons->priv->target_currency_combo), &iter);
+    update_currency_label(buttons);
+}
+
+
 static GtkWidget *
 load_mode(MathButtons *buttons, ButtonMode mode)
 {
@@ -706,6 +819,8 @@ load_mode(MathButtons *buttons, ButtonMode mode)
 
         if (button_data[i].tooltip)
             gtk_widget_set_tooltip_text(button, _(button_data[i].tooltip));
+      
+        atk_object_set_name (gtk_widget_get_accessible (button), button_data[i].widget_name);
 
         switch (button_data[i].class) {
         case NUMBER:
@@ -739,16 +854,26 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         name = g_strdup_printf("calc_%d_button", i);
         button = GET_WIDGET(builder, name);
         if (button) {
+            gchar buffer[7];
+            gint len;
+
             g_object_set_data(G_OBJECT(button), "calc_digit", GINT_TO_POINTER(i));
             set_tint(button, &buttons->priv->color_numbers, 1);
-            gtk_button_set_label(GTK_BUTTON(button), math_equation_get_digit_text(buttons->priv->equation, i));
+            len = g_unichar_to_utf8(math_equation_get_digit_text(buttons->priv->equation, i), buffer);
+            buffer[len] = '\0';
+            gtk_button_set_label(GTK_BUTTON(button), buffer);
         }
         g_free(name);
     }
     widget = GET_WIDGET(builder, "calc_numeric_point_button");
-    if (widget)
-        gtk_button_set_label(GTK_BUTTON(widget), math_equation_get_numeric_point_text(buttons->priv->equation));
-  
+    if (widget) {
+        gchar buffer[7];
+        gint len;
+        len = g_unichar_to_utf8(math_equation_get_numeric_point_text(buttons->priv->equation), buffer);
+        buffer[len] = '\0';
+        gtk_button_set_label(GTK_BUTTON(widget), buffer);
+    }
+
     widget = GET_WIDGET(builder, "calc_superscript_button");
     if (widget) {
         buttons->priv->superscript_toggles = g_list_append(buttons->priv->superscript_toggles, widget);
@@ -789,7 +914,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(buttons->priv->angle_combo), renderer, "text", 0);
 
         g_signal_connect(buttons->priv->angle_combo, "changed", G_CALLBACK(angle_unit_combobox_changed_cb), buttons);
-        g_signal_connect(buttons->priv->equation, "notify::angle-unit", G_CALLBACK(angle_unit_cb), buttons);
+        g_signal_connect(buttons->priv->equation, "notify::angle-units", G_CALLBACK(angle_unit_cb), buttons);
         angle_unit_cb(buttons->priv->equation, NULL, buttons);
     }
 
@@ -841,8 +966,41 @@ load_mode(MathButtons *buttons, ButtonMode mode)
 
     /* Setup financial functions */
     if (mode == FINANCIAL) {
+        GtkListStore *model;
+        GtkCellRenderer *renderer;
+
         load_finc_dialogs(buttons);
 
+        buttons->priv->source_currency_combo = GET_WIDGET(builder, "source_currency_combo");
+        buttons->priv->target_currency_combo = GET_WIDGET(builder, "target_currency_combo");
+        buttons->priv->currency_label = GET_WIDGET(builder, "currency_label");
+
+        model = gtk_list_store_new(1, G_TYPE_STRING);
+
+        for (i = 0; currency_names[i].short_name != NULL; i++) {
+            GtkTreeIter iter;
+
+            gtk_list_store_append(GTK_LIST_STORE(model), &iter);
+            gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, currency_names[i].short_name, -1);
+        }
+
+        gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->source_currency_combo), GTK_TREE_MODEL(model));
+        gtk_combo_box_set_model(GTK_COMBO_BOX(buttons->priv->target_currency_combo), GTK_TREE_MODEL(model));
+
+        renderer = gtk_cell_renderer_text_new();
+        gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(buttons->priv->source_currency_combo), renderer, TRUE);
+        gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(buttons->priv->source_currency_combo), renderer, "text", 0);
+        renderer = gtk_cell_renderer_text_new();
+        gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(buttons->priv->target_currency_combo), renderer, TRUE);
+        gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(buttons->priv->target_currency_combo), renderer, "text", 0);
+
+        g_signal_connect(buttons->priv->source_currency_combo, "changed", G_CALLBACK(source_currency_combo_changed_cb), buttons);
+        g_signal_connect(buttons->priv->target_currency_combo, "changed", G_CALLBACK(target_currency_combo_changed_cb), buttons);
+        g_signal_connect(buttons->priv->equation, "notify::source-currency", G_CALLBACK(source_currency_changed_cb), buttons);
+        g_signal_connect(buttons->priv->equation, "notify::target-currency", G_CALLBACK(target_currency_changed_cb), buttons);
+        source_currency_changed_cb(buttons->priv->equation, NULL, buttons);
+        target_currency_changed_cb(buttons->priv->equation, NULL, buttons);
+
         set_data(builder, "calc_finc_compounding_term_button", "finc_dialog", "ctrm_dialog");
         set_data(builder, "calc_finc_double_declining_depreciation_button", "finc_dialog", "ddb_dialog");
         set_data(builder, "calc_finc_future_value_button", "finc_dialog", "fv_dialog");
@@ -930,6 +1088,7 @@ math_buttons_get_programming_base(MathButtons *buttons)
 }
 
 
+void exponent_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 exponent_cb(GtkWidget *widget, MathButtons *buttons)
@@ -938,6 +1097,7 @@ exponent_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void subtract_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 subtract_cb(GtkWidget *widget, MathButtons *buttons)
@@ -946,6 +1106,7 @@ subtract_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void button_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 button_cb(GtkWidget *widget, MathButtons *buttons)
@@ -954,6 +1115,7 @@ button_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void solve_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 solve_cb(GtkWidget *widget, MathButtons *buttons)
@@ -962,6 +1124,7 @@ solve_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void clear_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 clear_cb(GtkWidget *widget, MathButtons *buttons)
@@ -970,6 +1133,7 @@ clear_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void delete_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 delete_cb(GtkWidget *widget, MathButtons *buttons)
@@ -978,6 +1142,7 @@ delete_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void undo_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 undo_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1089,6 +1254,7 @@ store_menu_cb(GtkMenuItem *menu, MathButtons *buttons)
 }
 
 
+void store_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 store_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1134,6 +1300,7 @@ recall_menu_cb(GtkMenuItem *menu, MathButtons *buttons)
 }
 
 
+void recall_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 recall_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1174,6 +1341,7 @@ recall_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void shift_left_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 shift_left_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1217,6 +1385,7 @@ shift_left_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void shift_right_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 shift_right_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1267,6 +1436,7 @@ insert_function_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void function_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 function_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1313,6 +1483,7 @@ function_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void factorize_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 factorize_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1321,6 +1492,7 @@ factorize_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void digit_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 digit_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1329,6 +1501,7 @@ digit_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void numeric_point_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 numeric_point_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1338,6 +1511,7 @@ numeric_point_cb(GtkWidget *widget, MathButtons *buttons)
 
 
 
+void finc_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 finc_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1350,6 +1524,7 @@ finc_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void insert_character_code_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 insert_character_code_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1358,6 +1533,7 @@ insert_character_code_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void finc_activate_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 finc_activate_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1384,6 +1560,7 @@ finc_activate_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void finc_response_cb(GtkWidget *widget, gint response_id, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 finc_response_cb(GtkWidget *widget, gint response_id, MathButtons *buttons)
@@ -1412,145 +1589,7 @@ finc_response_cb(GtkWidget *widget, gint response_id, MathButtons *buttons)
 }
 
 
-static void
-recalculate_currency(MathButtons *buttons, CurrencyTargetRow target)
-{
-    int upper_index, lower_index;
-
-    GtkComboBox *combo_upper = GTK_COMBO_BOX(gtk_builder_get_object(buttons->priv->financial_ui, "currency_type_upper"));
-    GtkComboBox *combo_lower = GTK_COMBO_BOX(gtk_builder_get_object(buttons->priv->financial_ui, "currency_type_lower"));
-    GtkSpinButton *spin_upper = GTK_SPIN_BUTTON(gtk_builder_get_object(buttons->priv->financial_ui, "currency_amount_upper"));
-    GtkSpinButton *spin_lower = GTK_SPIN_BUTTON(gtk_builder_get_object(buttons->priv->financial_ui, "currency_amount_lower"));
-
-    GtkTreeModel *model = gtk_combo_box_get_model(combo_upper);
-    GtkTreeIter iter;
-
-    if (!gtk_combo_box_get_active_iter(combo_upper, &iter))
-        return;
-    gtk_tree_model_get(model, &iter, 0, &upper_index, -1);
-
-    if (!gtk_combo_box_get_active_iter(combo_lower, &iter))
-        return;
-    gtk_tree_model_get(model, &iter, 0, &lower_index, -1);
-
-    if (target == CURRENCY_TARGET_LOWER) {
-        MPNumber input, output;
-        mp_set_from_double (gtk_spin_button_get_value(spin_upper), &input);
-        currency_convert(&input, upper_index, lower_index, &output);
-        if (!mp_is_zero(&output))
-            gtk_spin_button_set_value(spin_lower, mp_cast_to_double(&output));
-    } else {
-        MPNumber input, output;
-        mp_set_from_double (gtk_spin_button_get_value(spin_lower), &input);
-        currency_convert(&input, lower_index, upper_index, &output);
-        if (!mp_is_zero(&output))
-            gtk_spin_button_set_value(spin_upper, mp_cast_to_double(&output));
-    }
-}
-
-
-G_MODULE_EXPORT
-void
-currency_type_cb(GtkComboBox *combo, gpointer user_data, MathButtons *buttons)
-{
-    recalculate_currency(buttons, CURRENCY_TARGET_LOWER);
-}
-
-
-G_MODULE_EXPORT
-void
-currency_amount_cb (GtkSpinButton *spinbutton, gpointer user_data, MathButtons *buttons)
-{
-    recalculate_currency(buttons, GPOINTER_TO_INT(g_object_get_data(G_OBJECT(spinbutton), "target")));
-}
-
-static void
-setup_currency_rates(MathButtons *buttons)
-{
-    static int has_run = 0;
-    int i;
-    GtkListStore *currency_store;
-    GObject *currency_type;
-
-    if (has_run)
-        return;
-
-    if (currency_rates_needs_update()) {
-        GtkWidget *dialog = gtk_message_dialog_new(NULL, 0,
-                                        GTK_MESSAGE_INFO,
-                                        GTK_BUTTONS_YES_NO,
-                                        /* Translators: Title of the error dialog when prompting to download currency rates */
-                                        _("You don't have any recent currency rates. Should some be downloaded now?"));
-        int response = gtk_dialog_run(GTK_DIALOG(dialog));
-        gtk_widget_destroy(dialog);
-
-        if (response == GTK_RESPONSE_YES) {
-            if (!currency_download_rates()) {
-                dialog = gtk_message_dialog_new(NULL, 0,
-                                                GTK_MESSAGE_ERROR,
-                                                GTK_BUTTONS_OK,
-                                                /* Translators: Title of the error dialog when unable to download currency rates */
-                                                _("Currency rates could not be downloaded. You may receive inaccurate results, or you may not receive any results at all."));
-            }
-        }
-    }
-    currency_load_rates();
-
-    currency_type = gtk_builder_get_object(buttons->priv->financial_ui, "currency_type_upper");
-    currency_store = GTK_LIST_STORE(gtk_combo_box_get_model(
-        GTK_COMBO_BOX(currency_type)));
-
-    for (i = 0; currency_names[i].short_name; i++) {
-        GtkTreeIter iter;
-        int index;
-
-        if ((index = currency_get_index(currency_names[i].short_name)) < 0) {
-            continue;
-        }
-        gtk_list_store_append(currency_store, &iter);
-        gtk_list_store_set(currency_store, &iter,
-                           0, index,
-                           1, gettext(currency_names[i].long_name),
-                           -1);
-    }
-
-    has_run = 1;
-}
-
-
-G_MODULE_EXPORT
-void
-currency_cb(GtkWidget *widget, MathButtons *buttons)
-{
-    GtkDialog *win;
-    GtkSpinButton *c_amount_upper, *c_amount_lower;
-    MPNumber display_val;
-
-    setup_currency_rates(buttons);
-
-    win = GTK_DIALOG(gtk_builder_get_object(buttons->priv->financial_ui, "currency_dialog"));
-    c_amount_upper = GTK_SPIN_BUTTON(gtk_builder_get_object(buttons->priv->financial_ui, "currency_amount_upper"));
-    c_amount_lower = GTK_SPIN_BUTTON(gtk_builder_get_object(buttons->priv->financial_ui, "currency_amount_lower"));
-    if (math_equation_get_number(buttons->priv->equation, &display_val)) {
-        double start_val = mp_cast_to_double(&display_val);
-        gtk_spin_button_set_value(c_amount_upper, start_val);
-    }
-    gtk_widget_grab_focus(GTK_WIDGET(c_amount_upper));
-
-    if (gtk_dialog_run(win) == GTK_RESPONSE_OK) {
-        gchar *result;
-
-        result = g_strdup_printf("%.2f", gtk_spin_button_get_value(c_amount_lower));
-        mp_set_from_string(result, 10, &display_val);
-        g_free(result);
-
-        math_equation_set_number(buttons->priv->equation, &display_val);
-    }
-
-    gtk_widget_hide(GTK_WIDGET(win));
-}
-
-
+void character_code_dialog_response_cb(GtkWidget *dialog, gint response_id, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 character_code_dialog_response_cb(GtkWidget *dialog, gint response_id, MathButtons *buttons)
@@ -1581,6 +1620,7 @@ character_code_dialog_response_cb(GtkWidget *dialog, gint response_id, MathButto
 }
 
 
+void character_code_dialog_activate_cb(GtkWidget *entry, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 character_code_dialog_activate_cb(GtkWidget *entry, MathButtons *buttons)
@@ -1589,6 +1629,7 @@ character_code_dialog_activate_cb(GtkWidget *entry, MathButtons *buttons)
 }
 
 
+gboolean character_code_dialog_delete_cb(GtkWidget *dialog, GdkEvent *event, MathButtons *buttons);
 G_MODULE_EXPORT
 gboolean
 character_code_dialog_delete_cb(GtkWidget *dialog, GdkEvent *event, MathButtons *buttons)
@@ -1598,6 +1639,7 @@ character_code_dialog_delete_cb(GtkWidget *dialog, GdkEvent *event, MathButtons
 }
 
 
+gboolean bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event, MathButtons *buttons);
 G_MODULE_EXPORT
 gboolean
 bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event, MathButtons *buttons)
@@ -1608,6 +1650,7 @@ bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event, MathButtons *buttons)
 
 
 
+void set_superscript_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 set_superscript_cb(GtkWidget *widget, MathButtons *buttons)
@@ -1619,6 +1662,7 @@ set_superscript_cb(GtkWidget *widget, MathButtons *buttons)
 }
 
 
+void set_subscript_cb(GtkWidget *widget, MathButtons *buttons);
 G_MODULE_EXPORT
 void
 set_subscript_cb(GtkWidget *widget, MathButtons *buttons)
