@@ -23,7 +23,7 @@
 #include <libintl.h>
 
 #include "mp.h"
-#include "mp-internal.h"
+#include "mp-private.h"
 
 static int
 mp_compare_mp_to_int(const MPNumber *x, int i)
@@ -154,11 +154,10 @@ mpsin1(const MPNumber *x, MPNumber *z, int do_sin)
 }
 
 
-void
-mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+static void
+mp_sin_real(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
     int xs;
-    float rx = 0.0;
     MPNumber x_radians;
 
     /* sin(0) = 0 */
@@ -224,8 +223,8 @@ mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 }
 
 
-void
-mp_cos(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+static void
+mp_cos_real(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
     /* cos(0) = 1 */
     if (mp_is_zero(x)) {
@@ -251,6 +250,55 @@ mp_cos(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 }
 
 
+void
+mp_sin(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+{
+    if (mp_is_complex(x)) {
+        MPNumber x_real, x_im, z_real, z_im, t;
+
+        mp_real_component(x, &x_real);
+        mp_imaginary_component(x, &x_im);
+
+        mp_sin_real(&x_real, unit, &z_real);
+        mp_cosh(&x_im, &t);
+        mp_multiply(&z_real, &t, &z_real);
+
+        mp_cos_real(&x_real, unit, &z_im);
+        mp_sinh(&x_im, &t);
+        mp_multiply(&z_im, &t, &z_im);
+
+        mp_set_from_complex(&z_real, &z_im, z);
+    }
+    else
+       mp_sin_real(x, unit, z);
+}
+
+
+void
+mp_cos(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
+{
+    if (mp_is_complex(x)) {
+        MPNumber x_real, x_im, z_real, z_im, t;
+
+        mp_real_component(x, &x_real);
+        mp_imaginary_component(x, &x_im);
+
+        mp_cos_real(&x_real, unit, &z_real);
+        mp_cosh(&x_im, &t);
+        mp_multiply(&z_real, &t, &z_real);
+
+        mp_sin_real(&x_real, unit, &z_im);
+        mp_sinh(&x_im, &t);
+        mp_multiply(&z_im, &t, &z_im);
+        mp_invert_sign(&z_im, &z_im);
+
+        mp_set_from_complex(&z_real, &z_im, z);
+    }
+    else
+       mp_cos_real(x, unit, z);
+}
+
+
 void
 mp_tan(const MPNumber *x, MPAngleUnit unit, MPNumber *z)
 {
@@ -509,6 +557,7 @@ mp_tanh(const MPNumber *x, MPNumber *z)
     } else {
         mp_epowy(&t, &t);
         mp_add_integer(&t, 1, z);
+        mp_add_integer(&t, -1, &t);
         mp_divide(&t, z, z);
     }
 
